﻿/// IMPORTS ///
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using Blazorise.DataGrid;
using Blazorise.Extensions;
using Microsoft.AspNetCore.Components;
using Microsoft.Extensions.Localization;
using Microsoft.JSInterop;
using myBlazorApp.Models;
using myBlazorApp.Services;
using Syncfusion.Blazor.Grids;

namespace myBlazorApp.Components
{
    public partial class MyInventory
    {
        /// -------- INVENTORY PART -------- ///
       
        /// <summary>
        /// Get or Set the list of items of the inventory.
        /// </summary>
        [Parameter]
        public List<InventoryListItem> ItemsInventory { get; set; } = new List<InventoryListItem>();

        /// <summary>
        /// Get or Set the list of actions the user made.
        /// </summary>
        public ObservableCollection<InventoryAction> Actions { get; set; }

        /// <summary>
        /// Get or Set the item's name the user is dragging.
        /// </summary>
        public string CurrentDragItem { get; set; }

        /// <summary>
        /// Get or Set the item's index the user is dragging.
        /// </summary>
        public int CurrentDragIndex { get; set; }

        /// <summary>
        /// Get or Set the number of items the user is dragging.
        /// </summary>
        public int CurrentDragNumber { get; set; }

        /// <summary>
        /// Get or Set the item's stack size the user is dragging.
        /// </summary>
        public int CurrentDragStackSize { get; set; }

        /// <summary>
        /// Get or Set the InvntoryItem the user is dragging.
        /// </summary>
        public InventoryItem CurrentInventoryItem {get; set;} = new InventoryItem();

        ///-------- LIST PART -------- ///
        
        private List<Item> items = new List<Item>();

        private List<Item> full = new List<Item>();

        private List<Item> choix = new List<Item>();

        public string SearchText = "";

        private int totalItem;

        private int currentPage;

        private int pageSize;

        List<Item> Filtered = new List<Item>();

        List<Item> Sorted = new List<Item>();

        /// -------- DEPENDENCIES INJECTION -------- ///

        /// <summary>
        /// gets or sets the data service ( API or Local Stroage )
        /// </summary>
        [Inject]
        public IDataService DataService { get; set; }

        [Inject]
        public IWebHostEnvironment WebHostEnvironment { get; set; }

        [Inject]
        public NavigationManager NavigationManager { get; set; }

        private bool isSorted = false;

        [Inject]
        public IStringLocalizer<MyInventory> Localizer { get; set; }


        [Inject]
        internal IJSRuntime JavaScriptRuntime { get; set; }


        /// -------- METHODS -------- ///

        /// <summary>
        /// Constructor of the component.
        /// </summary>
        public MyInventory()
        {
            /// Creating new Actions list
            Actions = new ObservableCollection<InventoryAction>();
            /// subscribing the event "changement in the collection Actions" to the method OnActionsCollectionChanged.
            Actions.CollectionChanged += OnActionsCollectionChanged;
        }

        /// <summary>
        /// Method called when the collection "Actions" changes.
        /// Invoke the javascript script and call the API to save the inventory's data
        /// </summary>
        /// <param name="sender"> sender of the event </param>
        /// <param name="e"> event arguments </param>
        private async void OnActionsCollectionChanged(object? sender, NotifyCollectionChangedEventArgs e)
        {
            /// Invoke the js script
            JavaScriptRuntime.InvokeVoidAsync("MyInventory.AddActions", e.NewItems);

            /// call the dataservice if the action is "end", "drop" or "start"
            if (this.Actions.Last().Action == "End" || this.Actions.Last().Action == "Drop" || this.Actions.Last().Action == "Start")
            {
                string[] infos = new string[] { this.Actions.Last().ItemName, this.Actions.Last().Index.ToString(), this.Actions.Last().Number.ToString(), this.Actions.Last().StackSize.ToString() };
                await DataService.UpdateInventory(infos);
            }
        }

        /// <summary>
        /// Read data from the data service to display them in the inventory and in the data grid
        /// </summary>
        /// <param name="e"> Datagrid of items </param>
        /// <returns></returns>
        private async Task OnReadData(DataGridReadDataEventArgs<Item> e)
        {

            if (e.CancellationToken.IsCancellationRequested)
            {
                return;
            }

            if (!e.CancellationToken.IsCancellationRequested)
            {
                /// Read the items of the inventory
                ItemsInventory = await DataService.GetInventoryItems();
                totalItem = await DataService.Count();
                items = await DataService.List(e.Page, e.PageSize);
                full = await DataService.List(e.Page, totalItem);
                currentPage = e.Page;
                pageSize = e.PageSize;
                if (isSorted == false)
                {
                    choix = items;
                }
                else
                {
                    choix = SortList();
                    return;
                }
                if (SearchText.IsNullOrEmpty())
                {
                    choix = items;
                }
                else
                {
                    choix = choseList();
                    return;
                }
                StateHasChanged();
            }
        }

        private List<Item> choseList()
        {
            if (SearchText.IsNullOrEmpty())
            {
                choix = items;
                totalItem = full.Count();
                NavigationManager.NavigateTo("inventory", false);
            }
            else
            {
                if (Filtered.Count() < (currentPage - 1) * 10 + pageSize)
                {
                    pageSize = Filtered.Count() - (currentPage - 1) * 10;
                }
                choix = Filtered.GetRange((currentPage - 1) * 10, pageSize);
                totalItem = Filtered.Count();
            }
            StateHasChanged();
            NavigationManager.NavigateTo("inventory", false);
            return choix;
        }

        private void inputValue()
        {
            Filtered = full.Where(
               itm => itm.DisplayName.ToLower().Contains(SearchText.ToLower())).ToList();
            choseList();
        }

        private void OnPress()
        {
            if (isSorted == true)
            {
                isSorted = false;
            }
            else isSorted = true;
            SortList();
        }

        private List<Item> SortList()
        {
            if (isSorted == false)
            {
                choix = items;
                NavigationManager.NavigateTo("inventory", true);
            }
            else
            {
                if (Sorted.IsNullOrEmpty())
                {
                    Sorted = full;
                }
                Sorted.Sort((x, y) => string.Compare(x.DisplayName, y.DisplayName));
                if (Sorted.Count() < (currentPage - 1) * 10 + pageSize)
                {
                    pageSize = Sorted.Count() - (currentPage - 1) * 10;
                }
                choix = Sorted.GetRange((currentPage - 1) * 10, pageSize);
            }
            return choix;
        }
    }
}

